/**********************************************************************
 *
 * cgiwrapper.cpp -- output pages using the cgi protocol
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: cgiwrapper.cpp 146 1999-02-04 10:00:57Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/02/04 10:00:56  rjmcnab

   Developed the idea of an "action" and having them define the cgi arguments
   which they need and how those cgi arguments function.

   Revision 1.1  1999/02/04 01:16:17  rjmcnab

   Initial revision.

   Revision 1.5  1999/01/19 01:38:18  rjmcnab

   Made the source more portable.

   Revision 1.4  1999/01/12 01:51:04  rjmcnab

   Standard header.

 */


#include "gsdlconf.h"
#include "cgiwrapper.h"
#include <stdlib.h>

#if defined(GSDL_USE_OBJECTSPACE)
#  include <ospace/std/iostream>
#  include <ospace/std/fstream>
#elif defined(GSDL_USE_IOS_H)
#  include <iostream.h>
#  include <fstream.h>
#else
#  include <iostream>
#  include <fstream>
#endif

#include "cfgread.h"
#include "fileutil.h"

#ifdef USE_FASTCGI
#include "fcgiapp.h"
#endif

// Note: site.h would not be needed if we could
// dynamically find out gsdlhome.
#include "site.h"


#ifdef USE_FASTCGI
// used to output the text from receptionist
class fcgistreambuf : public streambuf {
public:
  fcgistreambuf ();
  int sync ();
  int overflow (int ch);
  int underflow () {return EOF;}
  
  void fcgisbreset() {fcgx_stream = NULL; other_ostream = NULL;};
  void set_fcgx_stream(FCGX_Stream *newone) {fcgx_stream=newone;};
  void set_other_ostream(ostream *newone) {other_ostream=newone;};
  
private:
  FCGX_Stream *fcgx_stream;
  ostream *other_ostream;
};

fcgistreambuf::fcgistreambuf() {
  fcgisbreset();
  if (base() == ebuf()) allocate();
  setp (base(), ebuf());
};

int fcgistreambuf::sync () {
  if ((fcgx_stream != NULL) &&
      (FCGX_PutStr (pbase(), out_waiting(), fcgx_stream) < 0)) {
    fcgx_stream = NULL;
  }

  if (other_ostream != NULL) {
    char *thepbase=pbase();
    for (int i=0;i<out_waiting();i++) (*other_ostream).put(thepbase[i]);
  }
  
  setp (pbase(), epptr());
  
  return 0;
}

int fcgistreambuf::overflow (int ch) {
  if (sync () == EOF) return EOF;
  if (ch != EOF) sputc (ch);
  return 0;
}

#endif


// returns true on success
static bool site_cfg_read (receptionist &recpt, const text_t &collection,
			   const text_t &filename, int &maxrequests) {
  text_tarray cfgline;
  char *cstr = filename.getcstr();
  ifstream confin (cstr);
  delete cstr;

  maxrequests = 10000;

  if (confin) {
    while (read_cfg_line(confin, cfgline) >= 0) {
      if (cfgline.size () >= 2) {
	if (cfgline[0] == "gsdlhome") {
	  if (collection.empty()) recpt.set_gsdlhome(cfgline[1]);
	  else recpt.set_gsdlhome(cfgline[1], collection);
	}
	else if (cfgline[0] == "httpimg") recpt.set_httpimg(cfgline[1]);
	else if (cfgline[0] == "gwcgi") recpt.set_gwcgi(cfgline[1]);
	else if (cfgline[0] == "maxrequests") {
	  maxrequests = cfgline[1].getint();
	  if (maxrequests < 1) maxrequests = 1;
	}
      }
    }
    confin.close ();
    return true;
  }
  return false;
}


static void page_errorconfigfile (const text_t &collection) {
  cout << "Content-type: text/html\n\n";

  cout << "<html>\n";
  cout << "<head>\n";
  cout << "<title>Error</title>\n";
  cout << "</head>\n";
  cout << "<body>\n";
  cout << "<h2>Oops!</h2>\n";
  cout << "The site.cfg configuration file could not be found. This file\n";
  cout << "should contain configuration information relating to this\n";
  cout << "site's setup. ";
  if (collection.empty()) {
    cout << "As this cgi script is not being run in collection specific mode,\n";
    cout << "the file should reside at " GSDL_GSDLHOME "/etc/site.cfg.\n";
  } else {
    char *cstr = collection.getcstr();
    cout << "As this cgi script is being run in collection specific mode,\n";
    cout << "the file can reside in " GSDL_GSDLHOME "/collect/" << cstr 
	 << "/etc/site.cfg or " GSDL_GSDLHOME "/etc/site.cfg.\n";
    delete cstr;
  }
  cout << "</body>\n";
  cout << "</html>\n";
}


static void page_errorinit () {
  cout << "Content-type: text/html\n\n";

  cout << "<html>\n";
  cout << "<head>\n";
  cout << "<title>Error</title>\n";
  cout << "</head>\n";
  cout << "<body>\n";
  cout << "<h2>Oops!</h2>\n";
  cout << "An error occurred during the initialisation of the Greenstone Digital\n";
  cout << "Library software. It is likely that the software has not been setup\n";
  cout << "correctly, consult " GSDL_GSDLHOME "/etc/initout.txt for more information.\n";
  cout << "</body>\n";
  cout << "</html>\n";
}


// cgiwrapper does everything necessary to output a page
// using the cgi protocol. If this is being run for a particular
// collection then "collection" should be set, otherwise it
// should equal "".
void cgiwrapper (receptionist &recpt, const text_t &collection) {
#ifdef USE_FASTCGI
  fcgistreambuf outbuf;
#endif

  // init stuff

  // set defaults
  int maxrequests = 10000;
  if (collection.empty()) recpt.set_gsdlhome(GSDL_GSDLHOME);
  else recpt.set_gsdlhome(GSDL_GSDLHOME, collection);
  recpt.set_httpimg("/gsdl/images");
  char *script_name = getenv("SCRIPT_NAME");
  if (script_name != NULL) recpt.set_gwcgi(script_name);
  else recpt.set_gwcgi("/cgi-bin/gw");

  // read in the site configuration file. Try to find it in the 
  // GSDLHOME/collect/collection-name/etc directory 
  // (if this is for a particular collection), and then GSDLHOME/etc.
  bool configfileread = false;
  text_t filename;
  if (!collection.empty()) {
    filename = filename_cat (GSDL_GSDLHOME, "collect");
    filename = filename_cat (filename, collection);
    filename = filename_cat (filename, "etc");
    filename = filename_cat (filename, "site.cfg");
    configfileread = site_cfg_read (recpt, collection, filename, maxrequests);
  }
  if (!configfileread) {
    filename = filename_cat (GSDL_GSDLHOME, "etc");
    filename = filename_cat (filename, "site.cfg");
    if (!site_cfg_read (recpt, collection, filename, maxrequests)) {
      // couldn't find the site configuration file
      page_errorconfigfile (collection);
      return;
    }
  }

  // initialise the library software
  ofstream initout (GSDL_GSDLHOME "/etc/initout.txt");
  if (!recpt.digest(initout)) {
    // an error occurred during the initialisation
    initout.close();
    page_errorinit();
    return;
  }
  initout.close();
  
  // find out whether this is being run as a cgi-script
  // or a fastcgi script
  int numrequests = 0;
#ifdef USE_FASTCGI
  int isfastcgi = !FCGX_IsCGI();
  FCGX_Stream *fcgiin, *fcgiout, *fcgierr;
  FCGX_ParamArray fcgienvp;
#else
  int isfastcgi = 0;
#endif

  // get the query string if it is not being run as a fastcgi
  // script
  text_t argstr = "";
  char *aURIStr;
  if (!isfastcgi) {
    aURIStr = getenv("QUERY_STRING");
    if (aURIStr != NULL) argstr = aURIStr;
    else {
      char cinURIStr[1024];
      cin >> cinURIStr;
      argstr = cinURIStr;
    }
    maxrequests = 1;
  }

  // Page-request loop. If this is not being run as a fastcgi
  // process then only one request will be processed and then
  // the process will exit.
  while (numrequests < maxrequests) {
#ifdef USE_FASTCGI
    if (isfastcgi) {
      if (FCGX_Accept(&fcgiin, &fcgiout, &fcgierr, &fcgienvp) < 0) break;
      aURIStr = FCGX_GetParam("QUERY_STRING", fcgienvp);
      if (aURIStr != NULL) argstr = aURIStr;
      else argstr = "";
    }
#endif

    // get output streams ready
#ifdef USE_FASTCGI
    outbuf.fcgisbreset ();
    if (isfastcgi) outbuf.set_fcgx_stream (fcgiout);
    else outbuf.set_other_ostream (&cout);
    ostream pageout (&outbuf);
#else
#define pageout cout
#endif
    ofstream errout (GSDL_GSDLHOME "/etc/errout.txt");
    cerr = errout;
    
    // get the query page
    recpt.produce_cgi_page (argstr, pageout, errout);
    
    // finish with the output streams
    errout.close();
#ifdef USE_FASTCGI
    if (isfastcgi) FCGX_Finish();
#endif

    numrequests++;
  }

  return;
}

