/**********************************************************************
 *
 * receptionist.cpp -- a web interface for the gsdl
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: receptionist.cpp 146 1999-02-04 10:00:57Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.3  1999/02/04 10:00:56  rjmcnab

   Developed the idea of an "action" and having them define the cgi arguments
   which they need and how those cgi arguments function.

   Revision 1.2  1999/02/04 01:17:27  rjmcnab

   Got it outputing something.


 */


#include "receptionist.h"
#include "fileutil.h"
#include <assert.h>


// this version of set_gsdlhome should be used if the receptionist
// is being run for multiple collections ("general" mode).
void receptionist::set_gsdlhome (const text_t &thegsdlhome) {
  gsdlhome = thegsdlhome;
  collectdir = thegsdlhome;
  collection = "";
}


// this version of set_gsdlhome should be used if the receptionist
// is being run for a single collection ("collection specific" mode).
void receptionist::set_gsdlhome (const text_t &thegsdlhome, const text_t &thecollection) {
  gsdlhome = thegsdlhome;
  collection = thecollection;

  // decide where collectdir is by searching for collect.cfg
  // look in $GSDLHOME/collect/collection-name/etc/collect.cfg and
  // then $GSDLHOME/etc/collect.cfg
  collectdir = filename_cat (gsdlhome, "collect");
  collectdir = filename_cat (collectdir, collection);
  text_t filename = filename_cat (collectdir, "etc");
  filename = filename_cat (filename, "collect.cfg");

  if (!file_exists(filename)) collectdir = gsdlhome;
}


// sets the http address of the images directory. This is used to
// speed up the access to the images which are a part of the general
// interface. If this is not set the interface will have to get the
// images via gwcgi which will be a lot slower (especially if the
// browser does not cache the images).
void receptionist::set_httpimg (const text_t &thehttpimg) {
  httpimg = thehttpimg;
}

// sets the http address of the gateway cgi program (ie. the program
// that contains this receptionist).
void receptionist::set_gwcgi (const text_t &thegwcgi) {
  gwcgi = thegwcgi;
}


// init should be called after setgsdhome has been called.
// It returns true on success and false on failure. If false is
// returned getpage should not be called (without producing
// meaningless output), instead an error page should be
// produced by the calling code.
bool receptionist::digest (ostream &logout) {
  // redirect the error output to logout
  //  disp.setlogout (&logout);

  // set default values for the configuration file
  //  cfg_info.defaultaction = "p";
  //  cfg_info.defaultpage = "about";
  //  cfg_info.defaultencoding = "w";

  // read in the configuration files etc/collect.cfg and index/build.cfg
  // entries in build.cfg should override those in collect.cfg
  //  filename = filename_cat (collectdir, "etc");
  //  filename = filename_cat (filename, "collect.cfg");
  //  cfg_read(filename);

  // load up the default macro files, the collection directory
  // is searched first for the file and then the main directory
  //  text_t colmacrodir = filename_cat (collectdir, "macros");
  //  text_t gsdlmacrodir = filename_cat (gsdlhome, "macros");
  //  text_tarray::iterator arrhere = cfg_info.macrofiles.begin();
  //  text_tarray::iterator arrend = cfg_info.macrofiles.end();
  //  while (arrhere != arrend) {
  //    filename = filename_cat (colmacrodir, *arrhere);
  //    if (!file_exists (filename)) {
  //      filename = filename_cat (gsdlmacrodir, *arrhere);
  //    }
  //    disp.loaddefaultmacros(filename);
  //    arrhere++;
  //  }

  //  srand(time(NULL));

  //  utf8outconvert.set_rzws(1);
  //  gboutconvert.set_rzws(1);

  //  return collect_init(collection);
  return true;
}


// produce_cgi_page will call parse_cgi_args, get_cgihead_info and 
// produce_content in the appropriate way to output a cgi header and
// the page content (if needed).
void receptionist::produce_cgi_page (const text_t &argstr, ostream &contentout, 
				     ostream &logout) {
  contentout << "Content-type: text/plain\n\nHello\n";
  contentout << flush;
}


// parse_cgi_args parses cgi arguments into an argument class.
// This function should be called for each page request.
void receptionist::parse_cgi_args (const text_t &argstr, cgiargsclass &args, 
				   ostream &logout) {
}


// get_cgihead_info determines the cgi header information for
// a set of cgi arguments. If response contains location then
// response_data contains the redirect address. If reponse
// contains content then reponse_data contains the content-type.
// Note that images can now be produced by the receptionist.
void receptionist::get_cgihead_info (cgiargsclass &args, response_t &response,
				     text_t &response_data, ostream &logout) {
}


// produce the page content
void receptionist::produce_content (cgiargsclass &args, ostream &contentout,
				    ostream &logout) {
}


// returns the compressed argument ("e") corresponding to the argument
// list. This can be used to save preferences between sessions.
text_t receptionist::get_compressed_arg (const cgiargsclass &args) {
  return "";
}
