/**********************************************************************
 *
 * receptionist.h -- a web interface for the gsdl
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: receptionist.h 146 1999-02-04 10:00:57Z rjmcnab $
 *
 *********************************************************************/


#ifndef RECEPTIONIST_H
#define RECEPTIONIST_H

#include "gsdlconf.h"
#include "text_t.h"
#include "cgiargs.h"

#ifndef MACROPRECEDENCE
#define MACROPRECEDENCE "style,collection,queryversion,version,language"
#endif


class receptionist {
public:
  receptionist () {}
  virtual ~receptionist() {}

  // this version of set_gsdlhome should be used if the receptionist
  // is being run for multiple collections ("general" mode).
  void set_gsdlhome (const text_t &thegsdlhome);

  // this version of set_gsdlhome should be used if the receptionist
  // is being run for a single collection ("collection specific" mode).
  void set_gsdlhome (const text_t &thegsdlhome, const text_t &thecollection);

  // sets the http address of the images directory. This is used to
  // speed up the access to the images which are a part of the general
  // interface. If this is not set the interface will have to get the
  // images via gwcgi which will be a lot slower (especially if the
  // browser does not cache the images).
  void set_httpimg (const text_t &thehttpimg);

  // sets the http address of the gateway cgi program (ie. the program
  // that contains this receptionist).
  void set_gwcgi (const text_t &thegwcgi);

  // digest should be called after setgsdhome has been called.
  // It returns true on success and false on failure. If false is
  // returned getpage should not be called (without producing
  // meaningless output), instead an error page should be
  // produced by the calling code.
  bool digest (ostream &logout);
  
  // There are two ways to produce a page. You can either call parse_cgi_args,
  // get_cgihead_info, and produce_content or you can just call produce_cgi_page
  // which will be satisfactory in most cases. You might want to call
  // parse_cgi_args, get_cgihead_info, and produce_content when you want to
  // call get_compressed_arg or interface directly with a web server for which
  // the standard header is inappropriate.

  // produce_cgi_page will call parse_cgi_args, get_cgihead_info and 
  // produce_content in the appropriate way to output a cgi header and
  // the page content (if needed).
  void produce_cgi_page (const text_t &argstr, ostream &contentout, 
			 ostream &logout);

  // parse_cgi_args parses cgi arguments into an argument class.
  // This function should be called for each page request.
  void parse_cgi_args (const text_t &argstr, cgiargsclass &args, ostream &logout);

  // response_t is used to inform the calling code what type of
  // cgi header it should produce
  // eventually this should reside in cgiutils.h
  enum response_t {location, content};

  // get_cgihead_info determines the cgi header information for
  // a set of cgi arguments. If response contains location then
  // response_data contains the redirect address. If reponse
  // contains content then reponse_data contains the content-type.
  // Note that images can now be produced by the receptionist.
  void get_cgihead_info (cgiargsclass &args, response_t &response,
			 text_t &response_data, ostream &logout);

  // produce the page content
  void produce_content (cgiargsclass &args, ostream &contentout,
			ostream &logout);

  // returns the compressed argument ("e") corresponding to the argument
  // list. This can be used to save preferences between sessions.
  text_t get_compressed_arg (const cgiargsclass &args);

protected:
  text_t gsdlhome;
  text_t collectdir; // will equal gsdlhome in 'general' mode
  text_t collection; // will equal "" in 'general' mode
  text_t httpimg;
  text_t gwcgi;


  
};



#endif
